#ifndef LLL_BASIS_H
#define LLL_BASIS_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * ===============
 * = lll_basis.h =
 * ===============
 *
 * history:
 * ========
 * $Id: lll_basis.h,v 1.1 1999/11/02 17:26:17 fv Exp $
 *
 * description:
 * ============
 * the only exported function LLL-reduces a basis of a lattice
 *
 * e-mail:
 * =======
 * frank.vallentin@math.uni-dortmund.de
 */

/*
 * computeLLLReducedBasis
 * ======================
 *
 * description:
 * ------------
 * computes a LLL-reduced basis using the standard scalarproduct
 *
 * input:
 * ------
 * long dim               - dimension of the lattice
 * double **basis         - the vectors of the basis as a dim x dim-matrix
 * double **reduced_basis - allocated dim x dim-matrix
 * int check              - flag. if check == 1, then the LLL-reduced conditions are
 *                          checked a posteriori
 *
 * output:
 * -------
 * double **reduced_basis - a LLL-reduced basis of the lattice
 */

extern void computeLLLReducedBasis(long dim,
				   double **basis,
				   double **reduced_basis,
				   int check);
#ifdef __cplusplus
}
#endif

#endif
