/*
 * ========
 * = sv.c =
 * ========
 *
 * history:
 * ========
 * $Id: sv.c,v 1.1 1999/11/02 17:25:35 fv Exp $
 *
 * description:
 * ============
 * test environment for shvec
 *
 * e-mail:
 * =======
 * frank.vallentin@math.uni-dortmund.de
 */

#include <ctype.h>
#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>

#include "shvec.h"
#include "la_support.h"

#define TRUE 1
#define FALSE 0


/*
 * main
 * ====
 */

int main(int argc,
	 char *argv[])
{
  int
    check,
    gram,
    read_1,
    read_2,
    r2,
    i,
    j,
    mode,
    number,
    dim,
    coset;
  double
    r1,
    bound,
    **gram_matrix;
  shvec_request
    request;
  shvec_info
    info;


  /* 
   * default values
   */

  read_1 = FALSE;
  read_2 = FALSE;
  r1 = 0;
  r2 = 0;

  bound = 0.0;
  number = 0;
  mode = SHVEC_MODE_BOUND;
  check = 0;
  coset = 0;
  gram = 0;


  /* 
   * user options
   */

  for (i = 1; i < argc; i++) {
    if (argv[i][0] == '-' && argv[i][1] == 'b')
      mode = SHVEC_MODE_BOUND;
    if (argv[i][0] == '-' && argv[i][1] == 's')
      mode = SHVEC_MODE_SHORTEST_VECTORS;
    if (argv[i][0] == '-' && argv[i][1] == 'm')
      mode = SHVEC_MODE_MINIMUM;
    if (argv[i][0] == '-' && argv[i][1] == 't')
      mode = SHVEC_MODE_THETA_SERIES;
    if (argv[i][0] == '-' && argv[i][1] == 'c')
      coset = 1;
    if (argv[i][0] == '-' && argv[i][1] == 'g')
      gram = 1;
    if (argv[i][0] == '-' && argv[i][1] == 'e')
      check = 1;
    if (argv[i][0] == '-' && argv[i][1] == 'h') {
      printf("usage: sv <in >out [-b x [y]] [-s [y]] [-m] [-t x] [-h]\n");
      printf("in: read dim and gram-matrix from in\n");
      printf("out: write output to out\n");
      printf("-g: print gram-matrix\n");
      printf("-b x [y]: print maximal y vectors v with (v, v) <= x\n");
      printf("-s [y]: print maximal y shortest vectors\n");
      printf("-m: determine the minimum\n");
      printf("-t x: compute the first x coefficients of thte theta-series\n");
      printf("-c: coset\n");
      printf("-e: check for errors\n");
      printf("-h: print this help\n");
      return 0;
    }
    if (isdigit(argv[i][0]) && read_1 && !read_2) {
      r2 = atoi(argv[i]);
      read_2 = TRUE;
    }
    if (isdigit(argv[i][0]) && !read_1) {
      r1 = atof(argv[i]);
      read_1 = TRUE;
    }
  }

  if (mode == SHVEC_MODE_BOUND) {
    bound = r1;
    number = r2;
  }  
  else if (mode == SHVEC_MODE_SHORTEST_VECTORS)
    number = r1;
  else if (mode == SHVEC_MODE_MINIMUM) {
  }
  else if (mode == SHVEC_MODE_THETA_SERIES) {
    if (r1 == 0) {
      printf("bound not specified\n");
      exit(1);
    }      
    bound = r1;
  }

  /*
   * input of gram matrix
   */

  scanf("%d", &dim);

  doubleMakeSquareMatrix(dim, &gram_matrix);

  for (i = 0; i < dim; i++)
    for (j = 0; j <= i; j++) {
      scanf("%lf", &gram_matrix[i][j]);
      gram_matrix[j][i] = gram_matrix[i][j];
    }

  if (mode == SHVEC_MODE_BOUND && bound == 0.0) {
    bound = gram_matrix[0][0];
    for (i = 1; i < dim; i++)
      if (bound < gram_matrix[i][i]) bound = gram_matrix[i][i];
  }

  if (gram)
    doublePrintGramMatrix(dim, gram_matrix);
      
  /*
   * initialization of request and info
   */

  request = (shvec_request) malloc(sizeof(struct shvec_request_struct));
  if (request == NULL) {
    fprintf(stderr, "sv.c: not enough memory!\n");
    exit(1);
  }
  initShvecReq(dim, gram_matrix, NULL, check, request);
  
  request->bound = bound;
  request->mode = mode;
  request->number = number;

  if (coset) {
    for (i = 0; i < dim; i++)
      scanf("%lf", &(request->coset[i]));
  }

  info = (shvec_info) malloc(sizeof(struct shvec_info_struct));
  if (info == NULL) {
    fprintf(stderr, "sv.c: not enough memory!\n");
    exit(1);
  }

  initShvecInfo(info);

  /* 
   * compute short vectors
   */

  computeShvec(request, info);

  /* 
   * output of results
   */

  if (mode == SHVEC_MODE_BOUND) {
    printf("%d\n", info->short_vectors_number);
    for (i = 0; i < info->short_vectors_number; i++) {
      printf("[%f]: ", info->short_vectors_norm[i]);
      intPrintVector(dim, info->short_vectors[i]);
    }
  }

  else if (mode == SHVEC_MODE_SHORTEST_VECTORS) {
    printf("%d\n", info->short_vectors_number);
    for (i = 0; i < info->short_vectors_number; i++) {
      printf("[%f]: ", info->short_vectors_norm[i]);
      intPrintVector(dim, info->short_vectors[i]);
    }
  }

  else if (mode == SHVEC_MODE_MINIMUM) {
    printf("Minimum: %f\n", info->minimum);
  }

  else if (mode == SHVEC_MODE_THETA_SERIES) {
    for (i = 0; i < info->theta_series_bound; i++) {
      if (i == 0)
	printf("%d + ", info->theta_series[i]);
      else if (i % 2 == 0 && info->theta_series[i] != 0)
	printf("%d q^{%d} + ", info->theta_series[i], i / 2);
      else if (i % 2 == 1 && info->theta_series[i] != 0)
	printf("%d q^{%d.5} + ", info->theta_series[i], i / 2);
    }
    i = info->theta_series_bound;
    if (i % 2 == 0)
      printf("%d q^{%d}\n", info->theta_series[i], i / 2);
    else
      printf("%d q^{%d.5}\n", info->theta_series[i], i / 2);
  }

  
  /*
   * clean up
   */
  
  destroyShvecInfo(info);
  destroyShvecReq(request);
  doubleDestroySquareMatrix(dim, &gram_matrix);
  free(request);
  free(info);


  return 0;
}
